import { StateField } from '@codemirror/state';
import { showTooltip } from '@codemirror/view';
import { signatureHelp } from '@neo4j-cypher/language-support';
function getTriggerCharacter(query, caretPosition) {
    let i = caretPosition - 1;
    let triggerCharacter = query.at(i);
    // Discard all space characters. Note that a space can be more than just a ' '
    while (/\s/.test(triggerCharacter) && i > 0) {
        i -= 1;
        triggerCharacter = query.at(i);
    }
    return triggerCharacter;
}
function getSignatureHelpTooltip(state, config) {
    let result = [];
    const schema = config.schema;
    const ranges = state.selection.ranges;
    const range = ranges.at(0);
    if (schema && ranges.length === 1 && range.from === range.to) {
        const caretPosition = range.from;
        const query = state.doc.toString();
        const triggerCharacter = getTriggerCharacter(query, caretPosition);
        if (triggerCharacter === '(' || triggerCharacter === ',') {
            const signatureHelpInfo = signatureHelp(query, schema, caretPosition);
            const activeSignature = signatureHelpInfo.activeSignature;
            const signatures = signatureHelpInfo.signatures;
            const activeParameter = signatureHelpInfo.activeParameter;
            if (activeSignature !== undefined &&
                activeSignature >= 0 &&
                activeSignature < signatures.length &&
                signatures[activeSignature].documentation !== undefined) {
                const signature = signatures[activeSignature];
                const parameters = signature.parameters;
                let doc = signature.documentation.toString();
                if (activeParameter >= 0 &&
                    activeParameter <
                        (signatures[activeSignature].parameters?.length ?? 0)) {
                    doc =
                        parameters[activeParameter].documentation.toString() + '\n\n' + doc;
                }
                result = [
                    {
                        pos: caretPosition,
                        above: true,
                        strictSide: true,
                        arrow: true,
                        create: () => {
                            const div = document.createElement('div');
                            const methodName = document.createElement('div');
                            const argPlusDescription = document.createElement('div');
                            const separator = document.createElement('hr');
                            const lineBreak = document.createElement('br');
                            div.append(...[methodName, separator, lineBreak, argPlusDescription]);
                            div.className = 'cm-tooltip-signature-help';
                            methodName.innerText = signature.label;
                            argPlusDescription.innerText = doc;
                            return { dom: div };
                        },
                    },
                ];
            }
        }
    }
    return result;
}
export function signatureHelpTooltip(config) {
    return StateField.define({
        create: (state) => getSignatureHelpTooltip(state, config),
        update(tooltips, tr) {
            if (!tr.docChanged && !tr.selection)
                return tooltips;
            return getSignatureHelpTooltip(tr.state, config);
        },
        provide: (f) => showTooltip.computeN([f], (state) => state.field(f)),
    });
}
//# sourceMappingURL=signatureHelp.js.map